package events;

import board.Board;
import game.Coordinate;
import game.Game;
import game.SetupGame;
import javafx.application.Platform;
import javafx.collections.ObservableList;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.input.MouseEvent;
import javafx.scene.shape.Circle;
import player.Player;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Created by danielford on 22/02/16.
 */
public class PlayerClick implements EventHandler {

    private Player player;
    private SetupGame state;
    private final static Logger LOGGER = Logger.getLogger(PlayerClick.class.getName());

    public PlayerClick(SetupGame gameState, Player player) {
        this.state = gameState;
        this.player = player;
    }

    public void handle(Event event) {

        Game game = state.getGame();
        Board board = state.getBoard();

        Node clickedNode = ((MouseEvent) event).getPickResult().getIntersectedNode(); // The clicked node
        Coordinate coordinates = (Coordinate) clickedNode.getUserData(); // Get the stored co-ordinate
        ObservableList<String> classes = clickedNode.getStyleClass(); // Get the CSS classes for the node

        try {

            if ( coordinates == null ) return;

            // Create the counter on the data layer
            board.createMove(coordinates, player.getPlayer());

            // Create the counter on the UI and progress to the next event
            Platform.runLater(() -> {
                createCounter( classes, clickedNode, game);
                state.nextEvent();
            });

        } catch (Error err) { LOGGER.log(Level.SEVERE, err.toString(), err); }

    }

    private void createCounter(ObservableList<String> classes, Node clickedNode, Game game) {

        classes.remove("empty");
        classes.add("taken");

        // Colour the node to the Player's colour
        ((Circle) clickedNode).setFill(player.getPlayerColor());


        // Remove the events for this player.
        game.getBoardUI().getStyleClass().remove("choosing");
        game.getStage().removeEventHandler(MouseEvent.MOUSE_CLICKED, this);

    }


}
