package game;

import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.geometry.Rectangle2D;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Paint;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import javafx.stage.Screen;
import javafx.stage.Stage;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import static javafx.scene.layout.GridPane.setMargin;

/**
 * Created by danielford on 05/05/16.
 */
public class ConfigGame {

    List<HashMap<String,String>> playerConfigs;
    private final Stage stage;
    private final SetupGame setupGame;

    private ColumnConstraints halfCol = new ColumnConstraints();
    private ColumnConstraints threeQuarterCol = new ColumnConstraints();
    private ColumnConstraints fullCol = new ColumnConstraints();

    public ConfigGame(Stage stage, SetupGame setupGame) {

        this.stage = stage;
        this.setupGame = setupGame;

        setupConfig();

    }

    private HashMap<String, String> defaultConfig() {
        HashMap<String, String> config = new HashMap<>();
        config.put("approach", "Human");
        return config;
    }

    private void setupConfig() {

        halfCol.setPercentWidth( 50 );
        threeQuarterCol.setPercentWidth( 75 );
        fullCol.setPercentWidth( 100 );

        // Initialise with two default human approaches.
        playerConfigs = new ArrayList<>();
        playerConfigs.add( defaultConfig() );
        playerConfigs.add( defaultConfig() );

        StackPane pane = new StackPane();
        pane.getStylesheets().add("resources/styles.css");
        BorderPane layout = layoutOptions();

        pane.getChildren().add(layout);

        stage.setResizable(false);
        stage.sizeToScene();
        stage.show();

        stage.setScene( new Scene( pane ) );
        stage.centerOnScreen();

    }

    private BorderPane layoutOptions() {

        BorderPane pane = new BorderPane();
        pane.getStyleClass().add("config-overlay");
        pane.setPrefSize(600, 600);

        Text titleText = new Text();

        // Customising and laying out of the Title Text
        titleText.setText("Gomoku");
        titleText.setFill(Paint.valueOf("WHITE"));
        titleText.setFont(Font.font("Arial", FontWeight.BOLD, 60));

        // Create the player screens in a GridPane
        GridPane middle = new GridPane();
        GridPane playerBoxes = new GridPane();
        GridPane boardConfig = boardControls();

        middle.getColumnConstraints().add(threeQuarterCol);

        playerBoxes.getColumnConstraints().addAll(halfCol, halfCol);

        playerBoxes.add(new PlayerConfig(1, this), 0, 0);
        playerBoxes.add(new PlayerConfig(2, this), 1, 0);

        middle.add(playerBoxes, 0, 0);
        middle.add(boardConfig, 0, 1);
        middle.setAlignment(Pos.TOP_CENTER);
        GridPane.setMargin(playerBoxes, new Insets(30, 0, 30, 0));
        GridPane.setMargin(boardConfig, new Insets(30, 0, 30, 0));

        pane.setTop( titleText );
        pane.setCenter( middle );

        BorderPane.setMargin( titleText, new Insets(20, 40, 20, 20));
        BorderPane.setMargin( playerBoxes, new Insets(20, 40, 20, 20));
        BorderPane.setAlignment(titleText, Pos.CENTER);
        BorderPane.setAlignment(playerBoxes, Pos.CENTER);



        return pane;

    }

    private GridPane boardControls() {

        GridPane pane = new GridPane();

        Text titleText = new Text();
        Text boardSizeText = new Text();
        Text scoreCountText = new Text();

        // Customising and laying out of the Title Text
        titleText.setText("Board Configurations");
        titleText.setFill(Paint.valueOf("WHITE"));
        titleText.setFont(Font.font("Arial", FontWeight.MEDIUM, 20));

        pane.add( titleText, 0 , 0 );
        GridPane.setMargin(titleText, new Insets(0, 0, 10, 0));

        boardSizeText.setText("Board Size");
        boardSizeText.setFill(Paint.valueOf("WHITE"));
        boardSizeText.setFont(Font.font("Arial", FontWeight.NORMAL, 16));
        TextField dimensionBox = inputBox();
        dimensionBox.setText("19");
        dimensionBox.setMaxWidth(135);

        pane.add( boardSizeText, 0 , 1 );
        pane.add( dimensionBox, 0, 2);
        GridPane.setMargin(dimensionBox, new Insets(5, 10, 0, 0));

        scoreCountText.setText("Winning Score");
        scoreCountText.setFill(Paint.valueOf("WHITE"));
        scoreCountText.setFont(Font.font("Arial", FontWeight.NORMAL, 15));
        TextField winningBox = inputBox();
        winningBox.setText("5");
        winningBox.setMaxWidth(135);

        pane.add( scoreCountText, 1 , 1 );
        pane.add(winningBox, 1, 2);
        GridPane.setMargin(winningBox, new Insets(5, 10, 0, 0));

        Button button = new Button();
        button.getStyleClass().addAll("btn", "btn-success");
        button.setText("Start Game");

        button.setOnAction((event) -> {
            setupGame.startGame( stage, Integer.parseInt(dimensionBox.getText()), Integer.parseInt(winningBox.getText()), playerConfigs);
        });

        pane.add(button, 2, 2);

        return pane;

    }

    private TextField inputBox() {
        return new TextField();
    }

    public void setPlayerConfig(int index, HashMap<String, String> config) {

        playerConfigs.set(index - 1, config);

    }




}
