package game;

import javafx.collections.FXCollections;
import javafx.event.ActionEvent;
import javafx.geometry.Insets;
import javafx.geometry.VPos;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.TextField;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.paint.Paint;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by danielford on 05/05/16.
 */
public class PlayerConfig extends GridPane {

    private int player;
    private ConfigGame configGame;
    private HashMap<String, String> config;
    private final Insets insets = new Insets(10, 0, 10, 0);
    private ChoiceBox<String> selection = new ChoiceBox<String>(FXCollections.observableArrayList("Human", "Heuristic", "MiniMax", "Iterative Deepening", "Monte Carlo"));

    public PlayerConfig(int player, ConfigGame configGame) {

        this.configGame = configGame;
        this.player = player;
        config = new HashMap<>();

        ColumnConstraints col = new ColumnConstraints();
        col.setPercentWidth(100);

        Text titleText = createText(20, "Player " + player);
        add( titleText, 0, 0);
        setValignment(titleText, VPos.CENTER);

        selection.setValue("Human");
        add(selection, 0, 1 );
        setMargin(selection, insets);



        selection.setOnAction(this::changeType);

    }

    private void changeType(ActionEvent actionEvent) {

        String selectionVal = selection.getValue();
        int selectionIndex = selection.getItems().indexOf( selectionVal );

        config.put("approach", selectionVal);
        subConfig( selectionIndex );

        configGame.setPlayerConfig(player, config);


    }

    private void subConfig( int selectedIndex ) {

        if ( getChildren().size() > 2 ) {
            getChildren().remove(2);
        }

        GridPane pane = new GridPane();

        if ( selectedIndex > 1 && selectedIndex < 4 ) {
            Text depthText = createText(16, "Max Depth");
            pane.add(depthText, 0, 2);
            pane.add(createInput("3", "maxDepth"), 0, 3);
            setMargin(depthText, insets);
        } else {
            config.remove("maxDepth");
        }

        if ( selectedIndex > 2 ) {
            Text timeText = createText(16, "Time Limit");
            pane.add(timeText, 0, 4);
            pane.add(createInput("15000", "maxTime"), 0, 5);
            setMargin(timeText, insets);
        } else {
            config.remove("maxTime");
        }

        if ( selectedIndex > 3 ) {
            Text movesText = createText(16, "Max Moves");
            pane.add(movesText, 0, 6);
            pane.add(createInput("20000", "maxMoves"), 0, 7);
            setMargin(movesText, insets);
        } else {
            config.remove("maxMoves");
        }

        add(pane, 0, 2);

    };

    private Text createText(int size, String label) {
        Text text = new Text();
        text.setFill(Paint.valueOf("WHITE"));
        text.setText(label);
        text.setFont(Font.font("Arial", FontWeight.MEDIUM, size));
        return text;
    }

    private TextField createInput(String defaultText, String key) {
        TextField field = new TextField();
        field.setText(defaultText);
        config.put(key, field.getText());

        field.textProperty().addListener((observable, oldValue, newValue) -> {
            config.put(key, newValue);
            configGame.setPlayerConfig(player, config);
        });

        return field;
    }


}
