package game;

import javafx.animation.RotateTransition;
import javafx.animation.Timeline;
import javafx.geometry.HPos;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.*;
import javafx.scene.shape.Circle;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import javafx.util.Duration;
import player.Player;

/**
 * Created by danielford on 24/02/16.
 */
public class PlayerTracker extends GridPane {

    private final Text thinking = new Text("Thinking...");
    private Circle playerPiece;
    private Text playerName;
    private final Image image = new Image("/resources/gear.png");
    private final ImageView imageView = new ImageView(image) ;


    public PlayerTracker() {

        playerPiece = new Circle();
        playerPiece.setRadius(12);

        playerName = new Text();


        setVgap(10);
        setMinWidth(125);

        Text trackerTitle = new Text();
        trackerTitle.setText("Player Turn");
        trackerTitle.setFont(Font.font("Arial", FontWeight.BOLD, 16));


        ColumnConstraints col1 = new ColumnConstraints();
        col1.setPercentWidth(100);

        getColumnConstraints().add(col1);


        setHalignment(trackerTitle, HPos.CENTER);
        add(trackerTitle, 0, 0);

        // Add and center the pieces
        setHalignment(playerPiece, HPos.CENTER);

        add(playerPiece, 0, 1);

        // Add the player name
        setHalignment(playerName, HPos.CENTER);
        add(playerName, 0, 2);


        getStyleClass().addAll("section", "first");

    }


    public void setCurrentPlayer(Player player) {

        playerPiece.setFill(player.getPlayerColor());
        playerName.setText("Player " + player.getPlayerIdentity());

        if ( player.getClass().getName().contains("Computer") ) {
            addThinking();
        }   else {
            removeThinking();
        }


    }

    private void addThinking() {

        imageView.setFitWidth(15);
        imageView.setFitHeight(15);

        getChildren().removeAll(thinking, imageView);

        setHalignment(thinking, HPos.CENTER);
        add(thinking, 0, 3);

        setHalignment(imageView, HPos.RIGHT);
        add(imageView, 0, 3);

        RotateTransition rt = new RotateTransition(Duration.millis(3000), imageView);
        rt.setByAngle(180);
        rt.setCycleCount(Timeline.INDEFINITE);

        rt.play();
    }

    private void removeThinking(){

        getChildren().removeAll(thinking, imageView);

    }


}
