package game;

import board.Board;
import javafx.concurrent.Task;
import javafx.geometry.Rectangle2D;
import javafx.stage.Screen;
import javafx.stage.Stage;
import player.Computer;
import player.Human;

import java.util.HashMap;
import java.util.List;
import java.util.Objects;

/**
 * Created by danielford on 25/02/16.
 */
public class SetupGame {

    private Board board;
    private Game game;

    public SetupGame(Stage stage) {

        new ConfigGame(stage, this);

    }

    public void startGame( Stage stage, int boardSize, int winningScore, List<HashMap<String, String>> configs ) {
        // Init the Data and UI representation of the Board

        board = new Board(boardSize, winningScore);
        game = new Game(stage, boardSize);


        // Create the two players
        for ( int i = 1; i < configs.size() + 1; i++ ) {
            String color = i == 1 ? "BLACK" : "WHITE";
            String approach = configs.get( i - 1).get("approach");
            if ( Objects.equals(approach, "Human") ) {
                board.createPlayer( new Human(i, color) );
            } else {
                Computer computer = new Computer(i, color, approach);
                computer.overrideConfig( configs.get( i - 1 ) );
                board.createPlayer( computer );
            }
        }


        // Set the first player on the board
        board.setCurrentPlayer( 0 );
        board.setFirstPlayer( board.getCurrentPlayer() );
        game.getPlayerTracker().setCurrentPlayer( board.getCurrentPlayer() );

        // Set the current player to the board and make them take their move
        createMoveTask();
    }


    /**
     * @return Game, the UI representation of the Board
     */
    public Game getGame() {
        return game;
    }

    /**
     * @return Board, The games Board
     */
    public Board getBoard() {
        return board;
    }

    // Switches the current player and checks if there has been a winner
    public void nextEvent() {

        // Switch the current Player
        board.setCurrentPlayer( board.getCurrentPlayerIndex() == 0 ? 1 : 0);

        String winner = board.checkWinner().getWinner();

        // Check if there is a winner, if there isn't continue
        if ( !Objects.equals(winner, "None") ) {
            game.setWinnerOverlay(winner);
        } else {
            game.getPlayerTracker().setCurrentPlayer( board.getCurrentPlayer() );
            createMoveTask();
        }

    }

    private void createMoveTask() {

        SetupGame gameState = this;

        Task task = new Task() {
            @Override
            protected Object call() throws Exception {
                board.getCurrentPlayer().takeMove( gameState );
                return null;
            }
        };

        new Thread(task).start();

        // Shows more errors than Task, useful for debugging
        // new Thread(new Runnable() {

        //    @Override
        //    public void run() {
        //        board.getCurrentPlayer().takeMove( gameState );
        //    }

        // }).start();
    }

}
