package minimax;

import board.Board;
import player.PositionAndScore;

import java.util.logging.Logger;

/**
 * Created by danielford on 06/04/16.
 */
public class DeepeningThread extends Thread {

    Board board;
    private int depth;
    private PositionAndScore bestSoFar;
    private boolean finished;
    private MiniMax minimax = null;
    private final static Logger LOGGER = Logger.getLogger(DeepeningThread.class.getName());

    public DeepeningThread(Board board, int depth) {
        this.board = board;
        this.depth = depth;
        finished = false;
    }

    @Override
    public void run() {

        PositionAndScore best = null;

        // Loop until finished or IterativeDeepening interrupts
        for ( ;; depth++ ) {

            // Create a new MiniMax instance and decide the move based on the new depth
            minimax = new MiniMax(board, board.getCurrentPlayer());
            PositionAndScore current = minimax.decideMove(depth);

            // Attempt to add the current to the best score
            if (best == null || current.getScore() > best.getScore()) {
                best = current;
                setBestSoFar( best );
            }

            // If the score - depth is greater or equal top 100, this means that we have definitely won?
            if ( best.getScore() - depth >= 100 || finished ) {
                break;
            }

        }

        // Attempt to set minimax bestSoFar as the best (in case best is null)
        setBestSoFar( minimax.getBestSoFar() );

        LOGGER.info("Final depth: " + depth + " and score: " + best.getScore());

        // Set finished to true, used by IterativeDeepening
        finished = true;

    }

    // Sets the bestSoFar to the best if it has no value, if best > current best also override.
    private void setBestSoFar( PositionAndScore best ) {
        if ( best != null && bestSoFar != null && best.getScore() > bestSoFar.getScore() ) {
            bestSoFar = best;
        } else {
            bestSoFar = best;
        }
    }

    // If minimax's bestSoFar is better than our own, return that
    public PositionAndScore getBestSoFar() {

        if ( bestSoFar != null && minimax != null && minimax.getBestSoFar() != null && minimax.getBestSoFar().getScore() > bestSoFar.getScore() ) {
            return minimax.getBestSoFar();
        } else {
            return bestSoFar;
        }

    }

    // Set finished to true and let minimax know that it should finish
    public void setFinished() {

        finished = true;
        minimax.setFinished();

    }

    // Retrieve finished, used in IterativeDeepening
    public boolean getFinished() { return finished; }


}
