import board.Board;
import board.WinState;
import org.junit.Test;
import player.Computer;
import player.Player;

import java.io.FileWriter;
import java.io.IOException;
import java.util.Objects;

/**
 * Created by danielford on 04/05/16.
 * Runs test on a 19 x 19 board
 */
public class ExperimentRunnerNine {

    private Board setupBoard() {
        return new Board(9, 4);
    }

    private void beginGame( Player player1, Player player2, int firstPlayer, FileWriter outputFile ) {

        Board board = setupBoard();

        board.createPlayer( player1 );
        board.createPlayer( player2 );

        board.setCurrentPlayer( firstPlayer );
        board.setFirstPlayer( board.getCurrentPlayer() );

        while ( !hasWinner(board.checkWinner()) ) {

            Computer player = (Computer) board.getCurrentPlayer();
            player.testCreateMove( board );
            board.setCurrentPlayer( board.getCurrentPlayerIndex() == 0 ? 1 : 0);

        }


        // Print out the winning Player results into the CSV
        Player winner = board.checkWinner().getPlayer();
        try {
            outputFile.append( player1.equals( winner ) ? "True": "False" );
            outputFile.append(',');
            outputFile.append( player2.equals( winner ) ? "True": "False" );
            outputFile.append(',');
            outputFile.append( board.getFirstPlayer().equals( player1 ) ? "True": "False" );
            outputFile.append('\n');
        } catch (IOException e) { e.printStackTrace(); }

    }

    private boolean hasWinner(WinState winner) {
        return winner != null && winner.getWinner() != null && !Objects.equals(winner.getWinner(), "None");
    }

    /**
     * Experiments of a Heuristic player vs a MiniMax player on a 19 x 19 board
     */

    private void testRunner(String firstType, String secondType, String document, int testNumber) throws IOException {

        Player player1 = new Computer(1, "BLACK", firstType);
        Player player2 = new Computer(2, "WHITE", secondType);
        int firstPlayer = 0;

        FileWriter writer = new FileWriter(document);

        writer.append(firstType);
        writer.append(',');
        writer.append(secondType);
        writer.append(',');
        writer.append("player1IsFirst");
        writer.append('\n');

        for( int i = 0; i < testNumber; i++ ){
            System.out.println("Iterations:" + i);
            beginGame(player1, player2, firstPlayer, writer);
            firstPlayer = firstPlayer == 0 ? 1 : 0;
        }

        writer.flush();
        writer.close();

    }

    @Test
    public void IterativeDeepeningMonteCarlo() throws IOException {

        testRunner("IterativeDeepening", "MonteCarlo-Heavy", "9x9IterativeDeepeningVSMonteCarlo.csv", 10);

    }



}
